<?xml version="1.0" encoding="UTF-8"?>
<project version="4">
  <component name="ProjectTasksOptions">
    <TaskOptions isEnabled="true">
      <option name="arguments" value="-f -c clion init --ide clion" />
      <option name="checkSyntaxErrors" value="true" />
      <option name="description" />
      <option name="exitCodeBehavior" value="NEVER" />
      <option name="fileExtension" value="ini" />
      <option name="immediateSync" value="false" />
      <option name="name" value="Monitor platformio.ini" />
      <option name="output" value="" />
      <option name="outputFilters">
        <array>
          <FilterInfo>
            <option name="description" value="" />
            <option name="name" value="PIO Conf" />
            <option name="regExp" value="$FILE_PATH$:^platofrmio" />
          </FilterInfo>
        </array>
      </option>
      <option name="outputFromStdout" value="false" />
      <option name="program" value="{{platformio_path}}" />
      <option name="scopeName" value="Project Files" />
      <option name="trackOnlyRoot" value="false" />
      <option name="workingDir" value="$PROJECT_DIR$" />
      <envs />
    </TaskOptions>
  </component>
</project>